import sys
import argparse

from glotter.run import run
from glotter.test import test
from glotter.download import download


def main():
    parser = argparse.ArgumentParser(
        prog='glotter',
        usage='''usage: glotter [-h] COMMAND

Commands:
  run         Run sources or group of sources. Use `glotter run --help` for more information.
  test        Run tests for sources or a group of sources. Use `glotter test --help` for more information.
  download    Download all the docker images required to run the tests
'''
    )
    parser.add_argument(
        'command',
        type=str,
        help='Subcommand to run',
        choices=['run', 'test', 'download']
    )
    args = parser.parse_args(sys.argv[1:2])
    commands = {
        'download': parse_download,
        'run': parse_run,
        'test': parse_test,
    }
    commands[args.command]()


def parse_download():
    parser = argparse.ArgumentParser(
        prog='glotter',
        description='Run a source or a group of sources. This command can run a language, a project'
                    'or a single source. Only one option may be specified.',
    )
    args = _parse_args_for_verb(parser)
    download(args)


def parse_run():
    parser = argparse.ArgumentParser(
        prog='glotter',
        description='Run a source or a group of sources. This command can run a language, a project'
                    'or a single source. Only one option may be specified.',
    )
    args = _parse_args_for_verb(parser)
    run(args)


def parse_test():
    parser = argparse.ArgumentParser(
        prog='glotter',
        description='Test a source or a group of sources. This command can test a language, a project'
                    'or a single source. Only one option may be specified.',
    )
    args = _parse_args_for_verb(parser)
    test(args)


def _parse_args_for_verb(parser):
    group = parser.add_mutually_exclusive_group()
    group.add_argument(
        '-s', '--source',
        metavar='SOURCE.EXT',
        type=str,
        help='source filename (not path) to run',
    )
    group.add_argument(
        '-p', '--project',
        metavar='PROJECT',
        type=str,
        help='project to run',
    )
    group.add_argument(
        '-l', '--language',
        metavar='LANGUAGE',
        type=str,
        help='language to run',
    )
    args = parser.parse_args(sys.argv[2:])
    return args


if __name__ == '__main__':
    main()



import os

import yaml

from glotter import testinfo
from glotter.settings import Settings
from glotter.containerfactory import ContainerFactory


class Source:
    """Metadata about a source file"""

    def __init__(self, name, path, test_info_string):
        """Initialize source

        :param name: filename including extension
        :param path: path to the file excluding name
        :param test_info_string: a string in yaml format containing testinfo for a directory
        """
        self._name = name
        self._path = path

        self._test_info = testinfo.TestInfo.from_string(test_info_string, self)

    @property
    def full_path(self):
        """Returns the full path to the source including filename and extension"""
        return os.path.join(self._path, self._name)

    @property
    def path(self):
        """Returns the path to the source excluding name"""
        return self._path

    @property
    def name(self):
        """Returns the name of the source excluding the extension"""
        return os.path.splitext(self._name)[0]

    @property
    def extension(self):
        """Returns the extension of the source"""
        return os.path.splitext(self._name)[1]

    @property
    def test_info(self):
        """Returns parsed TestInfo object"""
        return self._test_info

    def __repr__(self):
        return f'Source(name: {self.name}, path: {self.path})'

    def build(self, params=''):
        if self.test_info.container_info.build is not None:
            command = f'{self.test_info.container_info.build} {params}'
            result = self._container_exec(command)
            if result[0] != 0:
                raise RuntimeError(f'unable to build using cmd "{self.test_info.container_info.build} {params}":\n'
                                   f'{result[1].decode("utf-8")}')

    def run(self, params=None):
        """
        Run the source and return the output

        :param params: input passed to the source as it's run
        :return: the output of running the source
        """
        params = params or ''
        command = f'{self.test_info.container_info.cmd} {params}'
        result = self._container_exec(command)
        return result[1].decode('utf-8')

    def exec(self, command):
        """
        Run a command inside the container for a source

        :param command: command to run
        :return:  the output of the command as a string
        """
        result = self._container_exec(command)
        return result[1].decode('utf-8')

    def _container_exec(self, command):
        """
        Run a command inside the container for a source

        :param command: command to run
        :return:  the exit code and output of the command
        """
        container = ContainerFactory().get_container(self)
        return container.exec_run(
            cmd=command,
            detach=False,
            workdir='/src',
        )

    def cleanup(self):
        ContainerFactory().cleanup(self)


def get_sources(path):
    """
    Walk through a directory and create Source objects

    :param path: path to the directory through which to walk
    :return: a dict where the key is the ProjectType and the value is a list of all the Source objects of that project
    """
    sources = {k: [] for k in Settings().projects_enum}
    for root, dirs, files in os.walk(path):
        path = os.path.abspath(root)
        if "testinfo.yml" in files:
            with open(os.path.join(path, 'testinfo.yml'), 'r') as file:
                test_info_string = file.read()
            folder_info = testinfo.FolderInfo.from_dict(yaml.safe_load(test_info_string)['folder'])
            folder_project_names = folder_info.get_project_mappings(include_extension=True)
            for project_type, project_name in folder_project_names.items():
                if project_name in files:
                    source = Source(project_name, path, test_info_string)
                    sources[project_type].append(source)
    return sources



import os
import sys

from glotter.source import get_sources
from glotter.settings import Settings


def run(args):
    if args.language:
        _run_language(args.language)
    elif args.project:
        _run_project(args.project)
    elif args.source:
        _run_source(args.source)
    else:
        _run_all()


def _prompt_params(project_type):
    if not Settings().projects[project_type].requires_parameters:
        return ''
    return input(f'input parameters for "{project_type}": ')


def _build_and_run(source, params):
    print()
    print(f'Running "{source.name}{source.extension}"...')
    source.build()
    print(source.run(params))


def _error_and_exit(msg):
    print(msg)
    sys.exit(1)


def _run_all():
    sources_by_type = get_sources(Settings().source_root)
    for project_type, sources in sources_by_type.items():
        params = _prompt_params(project_type)
        for source in sources:
            _build_and_run(source, params)


def _run_language(language):
    sources_by_type = get_sources(path=os.path.join(Settings().source_root, language[0], language))
    if all([len(sources) <= 0 for _, sources in sources_by_type.items()]):
        _error_and_exit(f'No valid sources found for language: "{language}"')
    for project_type, sources in sources_by_type.items():
        for source in sources:
            params = _prompt_params(project_type)
            _build_and_run(source, params)


def _run_project(project):
    sources_by_type = get_sources(Settings().source_root)
    try:
        project_type = Settings().get_project_type_by_name(project)
        sources = sources_by_type[project_type]
        params = _prompt_params(project_type)
        for source in sources:
            _build_and_run(source, params)
    except KeyError:
        _error_and_exit(f'No valid sources found for project: "{project}"')


def _run_source(source):
    sources_by_type = get_sources(Settings().source_root)
    for project_type, sources in sources_by_type.items():
        for src in sources:
            if f'{src.name}{src.extension}'.lower() == source.lower():
                params = _prompt_params(project_type)
                _build_and_run(src, params)
                break
        else:  # If didn't break inner loop continue
            continue
        break  # Else break this loop as well
    else:
        _error_and_exit(f'Source "{source}" could not be found')



import docker
import shutil
import tempfile

from datetime import datetime, timedelta
from uuid import uuid4 as uuid

from glotter.singleton import Singleton


class ContainerFactory(metaclass=Singleton):

    def __init__(self, docker_client=None):
        """
        Initialize a ContainerFactory. This class is a singleton.

        :param docker_client: optionally set the docker client. Defaults to setting from the environment
        """
        self._containers = {}
        self._volume_dis = {}
        self._client = docker_client or docker.from_env()
        self._api_client = self._client.api

    def get_container(self, source):
        """
        Returns a running container for a give source. This will return an existing container if one exists
        or create a new one if necessary

        :param source: the source to use inside the container
        :return: a running container specific to the source
        """
        key = source.full_path

        tmp_dir = tempfile.mkdtemp()
        shutil.copy(source.full_path, tmp_dir)
        self._volume_dis[key] = tmp_dir

        image = self.get_image(source.test_info.container_info)
        volume_info = {tmp_dir: {'bind': '/src', 'mode': 'rw'}}
        if key not in self._containers:
            self._containers[key] = self._client.containers.run(
                image=image,
                name=f'{source.name}_{uuid().hex}',
                command='sleep 1h',
                working_dir='/src',
                volumes=volume_info,
                detach=True,
            )
        return self._containers[key]

    def get_image(self, container_info, quiet=False):
        """
        Pull a docker image

        :param container_info: metadata about the image to pull
        :param quiet: whether to print output while downloading
        :return: a docker image
        """
        images = self._client.images.list(name=f'{container_info.image}:{str(container_info.tag)}')
        if len(images) == 1:
            return images[0]
        if not quiet:
            print(f'Pulling {container_info.image}:{container_info.tag}... ', end='')
        last_update = datetime.now()
        for _ in self._api_client.pull(
                repository=container_info.image,
                tag=str(container_info.tag),
                stream=True,
                decode=True
        ):
            if datetime.now() - last_update > timedelta(seconds=5) and not quiet:
                print('... ', end='')
                last_update = datetime.now()
        if not quiet:
            print('done')
        images = self._client.images.list(name=f'{container_info.image}:{str(container_info.tag)}')
        if len(images) == 1:
            return images[0]

    def cleanup(self, source):
        """
        Cleanup docker container and temporary folder. Also remove both from their
        respective dictionaries

        :param source: source for determining what to cleanup
        """
        key = source.full_path

        self._containers[key].remove(v=True, force=True)
        shutil.rmtree(self._volume_dis[key], ignore_errors=True)

        del self._volume_dis[key]
        del self._containers[key]



from uuid import uuid4 as uuid

import pytest

from glotter import containerfactory
from glotter.source import Source
from glotter.testinfo import ContainerInfo
from test.unit.mockdocker import DockerMock


@pytest.fixture
def docker():
    docker_mock = DockerMock()
    yield docker_mock
    docker_mock.clear()


@pytest.fixture
def factory(docker):
    return containerfactory.ContainerFactory(docker_client=docker)


@pytest.fixture
def container_info():
    iid = uuid().hex
    return ContainerInfo(
        image=f'image_{iid}',
        tag=f'tag_{iid}',
        cmd=f'cmd_{iid}',
    )


@pytest.fixture
def test_info_string_no_build():
    return """folder:
  extension: ".py"
  naming: "underscore"

container:
  image: "python"
  tag: "3.7-alpine"
  cmd: "python {{ source.name }}{{ source.extension }}"
"""


@pytest.fixture
def test_info_string_with_build():
    return """folder:
  extension: ".go"
  naming: "hyphen"

container:
  image: "golang"
  tag: "1.12-alpine"
  build: "go build -o {{ source.name }} {{ source.name}}{{ source.extension }}"
  cmd: "./{{ source.name }}"
"""


@pytest.fixture
def source_no_build(test_info_string_no_build):
    iid = uuid().hex
    return Source(
        name=f'sourcename_{iid}',
        path=f'sourcepath_{iid}',
        test_info_string=test_info_string_no_build,
    )


@pytest.fixture
def source_with_build(test_info_string_with_build):
    iid = uuid().hex
    return Source(
        name=f'sourcename_{iid}',
        path=f'sourcepath_{iid}',
        test_info_string=test_info_string_with_build,
    )


@pytest.fixture
def no_io(monkeypatch):
    monkeypatch.setattr('tempfile.mkdtemp', lambda *args, **kwargs: 'TEMP_DIR')
    monkeypatch.setattr('shutil.copy', lambda *args, **kwargs: '')
    monkeypatch.setattr('shutil.rmtree', lambda *args, **kwargs: '')



import functools
from enum import Enum

import pytest

from glotter import Settings
from glotter.source import get_sources


def projects_enum(cls):
    settings = Settings()
    if not issubclass(cls, Enum):
        raise AttributeError('projects_enum must be called on a subclass of enum.Enum')
    settings.set_projects_enum(cls)
    return cls


def project_test(project_type):
    def decorator(func):
        @functools.wraps(func)
        def wrapper(func):
            Settings().add_test_mapping(project_type, func)
            return func
        return wrapper
    return decorator


def project_fixture(project_type):
    def decorator(func):
        @functools.wraps(func)
        def wrapper(func):
            sources = get_sources(Settings().source_root).get(project_type)
            return pytest.fixture(
                scope='module',
                params=sources,
                ids=[source.name + source.extension for source in sources],
            )
        return wrapper
    return decorator


import re
import os
import sys
import pytest

from glotter.source import get_sources
from glotter.settings import Settings


def test(args):
    if args.language:
        _run_language(args.language)
    elif args.project:
        _run_project(args.project)
    elif args.source:
        _run_source(args.source)
    else:
        _run_all()


def _error_and_exit(msg):
    print(msg)
    sys.exit(1)


def _get_tests(project_type, all_tests, src=None):
    test_function = Settings().get_test_mapping_name(project_type)
    if src is not None:
        filename = f'{src.name}{src.extension}'
        pattern = rf'^(\w/?)*\.py::{test_function}\[{filename}.*\]$'
    else:
        pattern = rf'^(\w/?)*\.py::{test_function}\[.+\]$'
    return [tst for tst in all_tests if re.fullmatch(pattern, tst) is not None]


def _run_all():
    _run_pytest_and_exit()


def _run_language(language):
    all_tests = _collect_tests()
    sources_by_type = get_sources(path=os.path.join('archive', language[0], language))
    if all([len(sources) <= 0 for _, sources in sources_by_type.items()]):
        _error_and_exit(f'No valid sources found for language: "{language}"')
    tests = []
    for project_type, sources in sources_by_type.items():
        for src in sources:
            tests += _get_tests(project_type, all_tests, src)
    _run_pytest_and_exit(*tests)


def _run_project(project):
    try:
        project_type = Settings().get_project_type_by_name(project)
        tests = _get_tests(project_type, _collect_tests())
        _run_pytest_and_exit(*tests)
    except KeyError:
        _error_and_exit(f'No valid sources found for project: "{project}"')


def _run_source(source):
    all_tests = _collect_tests()
    sources_by_type = get_sources('archive')
    for project_type, sources in sources_by_type.items():
        for src in sources:
            filename = f'{src.name}{src.extension}'
            if filename.lower() == source.lower():
                tests = _get_tests(project_type, all_tests, src)
                _run_pytest_and_exit(*tests)
                break
        else:  # If didn't break inner loop continue
            continue
        break  # Else break this loop as well
    else:
        _error_and_exit(f'Source "{source}" could not be found')


def _run_pytest_and_exit(*args):
    args = ['-v'] + list(args)
    code = pytest.main(args=args)
    sys.exit(code)


class TestCollectionPlugin:
    def __init__(self):
        self.collected = []

    def pytest_collection_modifyitems(self, items):
        for item in items:
            self.collected.append(item.nodeid)


def _collect_tests():
    print('============================= collect test totals ==============================')
    plugin = TestCollectionPlugin()
    pytest.main(['-qq', '--collect-only'], plugins=[plugin])
    return plugin.collected




import pytest

from glotter.project import Project, NamingScheme

project_scheme_permutation_map = [
    {
        "id": "singe_word_no_acronym",
        "words": ["word"],
        "acronyms": None,
        "schemes": {
            NamingScheme.hyphen: "word",
            NamingScheme.underscore: "word",
            NamingScheme.camel: "word",
            NamingScheme.pascal: "Word",
            NamingScheme.lower: "word",
        }},
    {
        "id": "multiple_words_no_acronym",
        "words": ["multiple", "words"],
        "acronyms": None,
        "schemes": {
            NamingScheme.hyphen: "multiple-words",
            NamingScheme.underscore: "multiple_words",
            NamingScheme.camel: "multipleWords",
            NamingScheme.pascal: "MultipleWords",
            NamingScheme.lower: "multiplewords",
        }},
    {
        "id": "single_acronym",
        "words": ["io"],
        "acronyms": ["IO"],
        "schemes": {
            NamingScheme.hyphen: "IO",
            NamingScheme.underscore: "IO",
            NamingScheme.camel: "io",
            NamingScheme.pascal: "IO",
            NamingScheme.lower: "io",
        }},
    {
        "id": "multiple_words_with_acronym_at_front",
        "words": ["io", "word"],
        "acronyms": ["IO"],
        "schemes": {
            NamingScheme.hyphen: "IO-word",
            NamingScheme.underscore: "IO_word",
            NamingScheme.camel: "ioWord",
            NamingScheme.pascal: "IOWord",
            NamingScheme.lower: "ioword",
        }},
    {
        "id": "multiple_words_with_acronym_in_middle",
        "words": ["words", "IO", "multiple"],
        "acronyms": ["IO"],
        "schemes": {
            NamingScheme.hyphen: "words-IO-multiple",
            NamingScheme.underscore: "words_IO_multiple",
            NamingScheme.camel: "wordsIOMultiple",
            NamingScheme.pascal: "WordsIOMultiple",
            NamingScheme.lower: "wordsiomultiple",
        }},
    {
        "id": "multiple_words_with_acronym_at_end",
        "words": ["multiple", "words", "IO"],
        "acronyms": ["IO"],
        "schemes": {
            NamingScheme.hyphen: "multiple-words-IO",
            NamingScheme.underscore: "multiple_words_IO",
            NamingScheme.camel: "multipleWordsIO",
            NamingScheme.pascal: "MultipleWordsIO",
            NamingScheme.lower: "multiplewordsio",
        }},
    {
        "id": "same_acronym_twice",
        "words": ["io", "word", "io"],
        "acronyms": ["io"],
        "schemes": {
            NamingScheme.hyphen: "IO-word-IO",
            NamingScheme.underscore: "IO_word_IO",
            NamingScheme.camel: "ioWordIO",
            NamingScheme.pascal: "IOWordIO",
            NamingScheme.lower: "iowordio",
        }},
    {
        "id": "multiple_acronyms",
        "words": ["io", "word", "ui"],
        "acronyms": ["ui", "io"],
        "schemes": {
            NamingScheme.hyphen: "IO-word-UI",
            NamingScheme.underscore: "IO_word_UI",
            NamingScheme.camel: "ioWordUI",
            NamingScheme.pascal: "IOWordUI",
            NamingScheme.lower: "iowordui",
        }},
    {
        "id": "multiple_acronyms_together",
        "words": ["word", "io", "ui"],
        "acronyms": ["ui", "io"],
        "schemes": {
            NamingScheme.hyphen: "word-IO-UI",
            NamingScheme.underscore: "word_IO_UI",
            NamingScheme.camel: "wordIOUI",
            NamingScheme.pascal: "WordIOUI",
            NamingScheme.lower: "wordioui",
        }
    },
]


def get_project_scheme_permutations():
    for perm in project_scheme_permutation_map:
        id = perm["id"]
        words = perm["words"]
        acronyms = perm["acronyms"]
        for scheme, expected in perm["schemes"].items():
            yield (id, words, acronyms, scheme, expected)


@pytest.mark.parametrize(("words", "acronyms", "scheme", "expected"),
                         [perm[1:] for perm in get_project_scheme_permutations()],
                         ids=[f'{perm[0]}_{perm[3]}' for perm in get_project_scheme_permutations()])
def test_get_project_name_by_scheme(words, acronyms, scheme, expected):
    project = Project(words, acronyms=acronyms)
    actual = project.get_project_name_by_scheme(scheme)
    assert actual == expected



import os
import sys

from glotter.source import get_sources
from glotter.settings import Settings
from glotter.containerfactory import ContainerFactory


def download(args):
    if args.language:
        _download_language(args.language)
    elif args.project:
        _download_project(args.project)
    elif args.source:
        _download_source(args.source)
    else:
        _download_all()


def _download_image_from_source(source):
    ContainerFactory().get_image(source.test_info.container_info)


def _error_and_exit(msg):
    print(msg)
    sys.exit(1)


def _download_all():
    sources_by_type = get_sources(Settings().source_root)
    for _, sources in sources_by_type.items():
        for source in sources:
            _download_image_from_source(source)


def _download_language(language):
    sources_by_type = get_sources(path=os.path.join(Settings().source_root, language[0], language))
    if all([len(sources) <= 0 for _, sources in sources_by_type.items()]):
        _error_and_exit(f'No valid sources found for language: "{language}"')
    for project_type, sources in sources_by_type.items():
        for source in sources:
            _download_image_from_source(source)


def _download_project(project):
    sources_by_type = get_sources(Settings().source_root)
    try:
        project_type = Settings.get_project_type_by_name(project)
        sources = sources_by_type[project_type]
        for source in sources:
            _download_image_from_source(source)
    except KeyError:
        _error_and_exit(f'No valid sources found for project: "{project}"')


def _download_source(source):
    sources_by_type = get_sources(Settings.source_root)
    for project_type, sources in sources_by_type.items():
        for src in sources:
            if f'{src.name}{src.extension}'.lower() == source.lower():
                _download_image_from_source(src)
                break
        else:  # If didn't break inner loop continue
            continue
        break  # Else break this loop as well
    else:
        _error_and_exit(f'Source "{source}" could not be found')



import os

from glotter import source

from test.integration.fixtures import tmp_dir, test_info_string_no_build, test_info_string_with_build, \
    patch_projects_enum, glotter_yml_projects, MockProjectEnum


def get_hello_world(language):
    return {
        'python':
            "def main():\n  print('Hello, world!')\n\nif __name__ == '__main__':\n  main()""",
        'go':
            'package main\n\nimport "fmt"\n\nfunc main() {\n\tfmt.Println("Hello, World!")\n}'
    }[language]


def create_files_from_list(files):
    for file_path, contents in files.items():
        dir = os.path.dirname(file_path)
        if not os.path.isdir(dir):
            os.makedirs(dir)
        with open(file_path, 'w') as file:
            file.write(contents)


def test_get_sources_when_no_testinfo(tmp_dir, test_info_string_no_build, test_info_string_with_build,
                                      patch_projects_enum):
    files = {
        os.path.join(tmp_dir, 'python', 'helloworld.py'): get_hello_world('python'),
        os.path.join(tmp_dir, 'go', 'hello-world.go'): get_hello_world('go'),
    }
    create_files_from_list(files)
    sources = source.get_sources(tmp_dir)
    assert not any(v for _, v in sources.items())


def test_get_sources(tmp_dir, test_info_string_no_build, test_info_string_with_build, patch_projects_enum,
                     glotter_yml_projects, monkeypatch):
    monkeypatch.setattr('glotter.settings.Settings.projects', glotter_yml_projects)
    files = {
        os.path.join(tmp_dir, 'python', 'testinfo.yml'): test_info_string_no_build,
        os.path.join(tmp_dir, 'python', 'hello_world.py'): get_hello_world('python'),
        os.path.join(tmp_dir, 'go', 'testinfo.yml'): test_info_string_with_build,
        os.path.join(tmp_dir, 'go', 'hello-world.go'): get_hello_world('go'),
    }
    create_files_from_list(files)
    sources = source.get_sources(tmp_dir)
    assert len(sources[MockProjectEnum.HelloWorld]) == 2
    assert not any(source_list for project_type, source_list in sources.items()
                   if project_type != MockProjectEnum.HelloWorld)



import os
import yaml

from enum import Enum
from warnings import warn

from glotter.project import Project, AcronymScheme
from glotter.containerfactory import Singleton
from glotter.projecthelpers import enum_from_string


class Settings(metaclass=Singleton):
    def __init__(self):
        self._projects_enum = None
        self._project_root = os.getcwd()
        self._parser = SettingsParser(self._project_root)
        self._source_root = self._parser.source_root or self._project_root
        self._test_mappings = {}

    @property
    def projects(self):
        return self._parser.projects

    @property
    def project_root(self):
        return self._project_root

    @property
    def source_root(self):
        return self._source_root

    @source_root.setter
    def source_root(self, value):
        self._source_root = value or self._project_root

    @property
    def test_mappings(self):
        return self._test_mappings

    @property
    def projects_enum(self):
        return self._projects_enum

    def get_test_mapping_name(self, project_type):
        return self._test_mappings[project_type].__name__

    def set_projects_enum(self, cls):
        if self._projects_enum is not None:
            raise AttributeError('Cannot set projects_enum more than once')
        if not issubclass(cls, Enum):
            raise AttributeError('projects_enum value must be a subclass of enum.Enum')
        self._projects_enum = cls
        self._parser.parse_projects_section()

    def add_test_mapping(self, project_type, func):
        if project_type not in self._test_mappings:
            self._test_mappings[project_type] = []
        self._test_mappings[project_type].append(func)

    @classmethod
    def get_project_type_by_name(cls, name):
        return enum_from_string(name, cls().projects_enum)


class SettingsParser:
    def __init__(self, project_root):
        self._project_root = project_root
        self._yml_path = None
        self._yml = None
        self._acronym_scheme = None
        self._projects = None
        self._source_root = None
        self._yml_path = self._locate_yml()
        if self._yml_path is not None:
            self._yml = self._parse_yml()
            self.parse_settings_section()
        else:
            warn(f'.glotter.yml not found in directory "{project_root}"')

    def parse_settings_section(self):
        if self._yml is not None:
            self._acronym_scheme = self._parse_acronym_scheme()
            self._source_root = self._parse_source_root()

    def parse_projects_section(self):
        if self.yml is not None:
            self._projects = self._parse_projects()

    @property
    def project_root(self):
        return self._project_root

    @property
    def yml_path(self):
        return self._yml_path

    @property
    def yml(self):
        return self._yml

    @property
    def source_root(self):
        return self._source_root

    @property
    def acronym_scheme(self):
        return self._acronym_scheme

    @property
    def projects(self):
        return self._projects

    def _parse_acronym_scheme(self):
        if 'settings' not in self._yml or 'acronym_scheme' not in self._yml['settings']:
            return

        scheme = self._yml['settings']['acronym_scheme'].lower()
        return AcronymScheme[scheme]

    def _parse_source_root(self):
        if 'settings' not in self._yml or 'source_root' not in self._yml['settings']:
            return

        path = self._yml['settings']['source_root']
        if os.path.isabs(path):
            return path

        yml_dir = os.path.dirname(self._yml_path)
        return os.path.abspath(os.path.join(yml_dir, path))

    def _parse_projects(self):
        if Settings().projects_enum is None:
            warn(f'Projects enum has not been set')
            return None

        projects = {}
        if 'projects' in self._yml:
            for k, v in self._yml['projects'].items():
                try:
                    project_name = enum_from_string(name=k, enum_cls=Settings().projects_enum)
                    project = Project(
                        words=v.get('words'),
                        requires_parameters=v.get('requires_parameters'),
                        acronyms=v.get('acronyms'),
                        acronym_scheme=v.get('acronym_scheme') or self._acronym_scheme,
                    )
                    projects[project_name] = project
                except KeyError:
                    warn(f'.glotter.yml contains a project that is not found in the projects_enum: "{k}"')

        return projects

    def _parse_yml(self):
        with open(self._yml_path, 'r') as f:
            contents = f.read()

        return yaml.safe_load(contents)

    def _locate_yml(self):
        for root, dirs, files in os.walk(self._project_root):
            if '.glotter.yml' in files:
                path = os.path.abspath(root)
                return os.path.join(path, '.glotter.yml')

        return None



import os
import pytest

from glotter.settings import SettingsParser
from glotter.project import AcronymScheme

from test.integration.fixtures import tmp_dir, glotter_yml, glotter_yml_projects, patch_projects_enum


def setup_settings_parser(tmp_dir, path, contents):
    os.makedirs(os.path.dirname(path), exist_ok=True)
    with open(path, 'w') as f:
        f.write(contents)
    return SettingsParser(tmp_dir)


def test_locate_yml_when_glotter_yml_does_not_exist(tmp_dir):
    settings_parser = SettingsParser(tmp_dir)
    settings_parser.parse_settings_section()
    assert settings_parser.yml_path is None


def test_locate_yml_when_glotter_yml_does_exist(tmp_dir, glotter_yml, patch_projects_enum):
    expected = os.path.join(tmp_dir, '.glotter.yml')
    settings_parser = setup_settings_parser(tmp_dir, expected, glotter_yml)
    settings_parser.parse_settings_section()
    assert settings_parser.yml_path == expected


def test_locate_yml_when_glotter_yml_is_not_at_root(tmp_dir, glotter_yml, patch_projects_enum):
    expected = os.path.join(tmp_dir, 'this', 'is', 'a', 'few', 'levels', 'deeper', '.glotter.yml')
    settings_parser = setup_settings_parser(tmp_dir, expected, glotter_yml)
    settings_parser.parse_settings_section()
    assert settings_parser.yml_path == expected


@pytest.mark.parametrize(('scheme_str', 'expected'),
                         [
                             ('upper', AcronymScheme.upper),
                             ('lower', AcronymScheme.lower),
                             ('two_letter_limit', AcronymScheme.two_letter_limit),
                         ])
def test_parse_acronym_scheme(scheme_str, expected, tmp_dir):
    glotter_yml = f'settings:\n  acronym_scheme: "{scheme_str}"'
    path = os.path.join(tmp_dir, '.glotter.yml')
    settings_parser = setup_settings_parser(tmp_dir, path, glotter_yml)
    settings_parser.parse_settings_section()
    assert settings_parser.acronym_scheme == expected


def test_parses_source_root_when_path_absolute(tmp_dir):
    expected = os.path.abspath(os.path.join(tmp_dir, 'subdir'))
    os.makedirs(expected)
    glotter_yml = f'settings:\n  source_root: "{expected}"'
    path = os.path.join(tmp_dir, '.glotter.yml')
    settings_parser = setup_settings_parser(tmp_dir, path, glotter_yml)
    settings_parser.parse_settings_section()
    assert settings_parser.source_root == expected


def test_parses_source_root_when_path_relative(tmp_dir):
    expected = os.path.abspath(os.path.join(tmp_dir, 'src'))
    os.makedirs(expected)
    glotter_yml = f'settings:\n  source_root: "../src"'
    path = os.path.join(tmp_dir, 'subdir', '.glotter.yml')
    settings_parser = setup_settings_parser(tmp_dir, path, glotter_yml)
    settings_parser.parse_settings_section()
    assert settings_parser.source_root == expected


def test_parse_projects_when_no_projects_enum(tmp_dir, glotter_yml):
    path = os.path.join(tmp_dir, '.glotter.yml')
    settings_parser = setup_settings_parser(tmp_dir, path, glotter_yml)
    settings_parser.parse_settings_section()
    assert settings_parser.projects is None


def test_parse_projects_when_no_projects(tmp_dir, patch_projects_enum):
    glotter_yml = f'settings:\n  acronym_scheme: "upper"'
    path = os.path.join(tmp_dir, '.glotter.yml')
    settings_parser = setup_settings_parser(tmp_dir, path, glotter_yml)
    settings_parser.parse_projects_section()
    assert settings_parser.projects == {}


def test_parse_projects_when_no_projects_or_projects_enum(tmp_dir):
    glotter_yml = f'settings:\n  acronym_scheme: "upper"'
    path = os.path.join(tmp_dir, '.glotter.yml')
    settings_parser = setup_settings_parser(tmp_dir, path, glotter_yml)
    settings_parser.parse_projects_section()
    assert settings_parser.projects is None


def test_parse_projects(tmp_dir, glotter_yml, glotter_yml_projects, patch_projects_enum):
    path = os.path.join(tmp_dir, '.glotter.yml')
    settings_parser = setup_settings_parser(tmp_dir, path, glotter_yml)
    settings_parser.parse_projects_section()
    assert settings_parser.projects == glotter_yml_projects


def test_parse_projects_when_yml_contains_projects_not_in_enum(tmp_dir, glotter_yml, glotter_yml_projects,
                                                               patch_projects_enum, monkeypatch):
    path = os.path.join(tmp_dir, '.glotter.yml')
    glotter_yml += """
  nonexistantproject:
    words:
      - "nonexistant"
      - "project"
    requires_parameters: false
"""
    settings_parser = setup_settings_parser(tmp_dir, path, glotter_yml)
    settings_parser.parse_projects_section()
    assert settings_parser.projects == glotter_yml_projects



import pytest

from uuid import uuid4 as uuid
from glotter.testinfo import ContainerInfo, FolderInfo, TestInfo
from glotter.project import NamingScheme


@pytest.mark.parametrize('build', [uuid().hex, None], ids=['with_build', 'without_build'])
def test_container_info_from_dict(build):
    dct = {
        'image': uuid().hex,
        'tag': uuid().hex,
        'cmd': uuid().hex,
    }
    expected = ContainerInfo(
        image=dct['image'],
        tag=dct['tag'],
        cmd=dct['cmd'],
        build=build
    )
    if build is not None:
        dct['build'] = build
    info = ContainerInfo.from_dict(dct)
    assert info == expected


def test_folder_info_from_dict():
    dct = {
        'extension': '.py',
        'naming': 'underscore'
    }
    expected = FolderInfo(
        extension=dct['extension'],
        naming=dct['naming'],
    )
    info = FolderInfo.from_dict(dct)
    assert info == expected


def test_test_info_from_dict():
    container_info_dict = {
        'image': uuid().hex,
        'tag': uuid().hex,
        'cmd': uuid().hex,
    }
    ci = ContainerInfo(**container_info_dict)
    folder_info_dict = {
        'extension': '.py',
        'naming': 'underscore'
    }
    fi = FolderInfo(**folder_info_dict)
    test_info = TestInfo.from_dict({
        'container': container_info_dict,
        'folder': folder_info_dict
    })
    expected = TestInfo(container_info=ci, file_info=fi)
    assert test_info == expected



from uuid import uuid4 as uuid


class ContainerExec:
    def __init__(self, cmd, attributes):
        self.cmd = cmd
        self._attributes = attributes

    def __getitem__(self, key):
        return self._attributes[key]


class Container:
    def __init__(self, image, name, attributes):
        self.image = image
        self.name = name
        self._attributes = attributes
        self.removed = False
        self.execs = []

    def __getitem__(self, key):
        return self._attributes[key]

    def remove(self, *args, **kwargs):
        self.removed = True

    def exec_run(self, cmd, **kwargs):
        self.execs.append(ContainerExec(cmd, kwargs))
        return 0, 'executed'.encode('utf-8')


class Containers:
    container_list = {}

    @classmethod
    def run(cls, image, **kwargs):
        name = kwargs['name'] if 'name' in kwargs else uuid().hex
        info = Container(image, name, kwargs)
        if name not in cls.container_list:
            cls.container_list[name] = info
            return cls.container_list[name]
        raise EnvironmentError('Container exists')

    @classmethod
    def clear(cls):
        cls.container_list = {}


class Images:
    image_list = []

    @classmethod
    def add_image(cls, name):
        if name not in cls.image_list:
            Images.image_list.append(name)

    @classmethod
    def list(cls, name=None, **kwargs):
        if name and name in cls.image_list:
            return [name]

        return cls.image_list

    @classmethod
    def clear(cls):
        cls.image_list = []


class DockerApi:
    @staticmethod
    def pull(repository, **kwargs):
        tag = kwargs.get('tag') or 'latest'
        Images.add_image(f'{repository}:{tag}')
        return Images.list(f'{repository}:{tag}')


class DockerMock:
    containers = Containers
    images = Images
    api = DockerApi

    @classmethod
    def clear(cls):
        cls.images.clear()
        cls.containers.clear()


from enum import Enum, auto


class NamingScheme(Enum):
    hyphen = auto()
    underscore = auto()
    camel = auto()
    pascal = auto()
    lower = auto()


class AcronymScheme(Enum):
    lower = auto()
    upper = auto()
    two_letter_limit = auto()


class Project:
    def __init__(self, words, requires_parameters=False, acronyms=None, acronym_scheme=None):
        self._words = words
        self._requires_parameters = requires_parameters
        self._acronyms = [acronym.upper() for acronym in acronyms] if acronyms else []
        self._acronym_scheme = acronym_scheme or AcronymScheme.upper

    @property
    def words(self):
        return self._words

    @property
    def requires_parameters(self):
        return self._requires_parameters

    @property
    def acronyms(self):
        return self._acronyms

    @property
    def acronym_scheme(self):
        return self._acronym_scheme

    def get_project_name_by_scheme(self, naming):
        """
        gets a project name for a specific naming scheme

        :param naming: the naming scheme
        :return: the project type formatted by the directory's naming scheme
        """
        try:
            return {
                NamingScheme.hyphen: self._as_hyphen(),
                NamingScheme.underscore: self._as_underscore(),
                NamingScheme.camel: self._as_camel(),
                NamingScheme.pascal: self._as_pascal(),
                NamingScheme.lower: self._as_lower(),
            }[naming]
        except KeyError as e:
            raise KeyError(f"Unknown naming scheme '{naming}'", e)

    def _as_hyphen(self):
        return '-'.join(map(self._try_as_acronym, self._words))

    def _as_underscore(self):
        return '_'.join(map(self._try_as_acronym, self._words))

    def _as_camel(self):
        return self._words[0].lower() + ''.join([self._try_as_acronym(word.title()) for word in self._words[1:]])

    def _as_pascal(self):
        return ''.join([self._try_as_acronym(word.title()) for word in self._words])

    def _as_lower(self):
        return ''.join([word.lower() for word in self._words])

    def _is_acronym(self, word):
        return word.upper() in self._acronyms

    def _try_as_acronym(self, word):
        if self._is_acronym(word):
            if self._acronym_scheme == AcronymScheme.upper:
                return word.upper()
            elif self._acronym_scheme == AcronymScheme.lower:
                return word.lower()
            elif self._acronym_scheme == AcronymScheme.two_letter_limit:
                if len(word) <= 2:
                    return word.upper()
        return word

    def __eq__(self, other):
        return self._words == other._words and \
               self._requires_parameters == other._requires_parameters and \
               self._acronyms == other._acronyms and \
               self._acronym_scheme == other._acronym_scheme



import pytest

from glotter.test import _get_tests

list_of_tests = [
    'test/projects/test_even_odd.py::test_even_odd_invalid[even-odd.c-no input]',
    'test/projects/test_even_odd.py::test_even_odd_invalid[even-odd.c-empty input]',
    'test/projects/test_even_odd.py::test_even_odd_invalid[even-odd.c-invalid input: not a number]',
    'test/projects/test_even_odd.py::test_even_odd_valid[EvenOdd.cs-sample input: even]',
    'test/projects/test_even_odd.py::test_even_odd_valid[EvenOdd.cs-sample input: odd]',
    'test/projects/test_even_odd.py::test_even_odd_valid[EvenOdd.cs-sample input: negative even]',
    'test/projects/test_even_odd.py::test_even_odd_valid[EvenOdd.cs-sample input: negative odd]',
    'test/projects/test_even_odd.py::test_even_odd_invalid[EvenOdd.cs-no input]',
    'test/projects/test_factorial.py::test_factorial_valid[factorial.hs-sample input: four]',
    'test/projects/test_factorial.py::test_factorial_valid[factorial.hs-sample input: eight]',
    'test/projects/test_factorial.py::test_factorial_valid[factorial.hs-sample input: ten]',
    'test/projects/test_factorial.py::test_factorial_invalid[factorial.hs-sample input: zero]',
    'test/projects/test_factorial.py::test_factorial_invalid[factorial.hs-sample input: one]',
    'test/projects/test_factorial.py::test_factorial_invalid[factorial.hs-sample input: four]',
]


@pytest.mark.parametrize('test_function',
                         [
                             'test_even_odd_invalid',
                             'test_even_odd_valid',
                             'test_factorial_invalid',
                             'test_factorial_valid',
                         ])
def test_get_tests(test_function, monkeypatch):
    project_type = 'project_type'
    monkeypatch.setattr('glotter.settings.Settings.get_test_mapping_name', lambda *args, **kwargs: test_function)
    actual = _get_tests(project_type, list_of_tests)
    for t in list_of_tests:
        if test_function in t:
            assert t in actual





from test.unit.mockdocker import Containers, Images
from test.unit.fixtures import factory, container_info, source_no_build, docker, test_info_string_no_build, no_io


def test_get_image_returns_image(factory, container_info):
    result = factory.get_image(container_info, quiet=True)
    assert result == f'{container_info.image}:{container_info.tag}'


def test_get_image_downloads_image_when_not_found(factory, container_info):
    factory.get_image(container_info, quiet=True)
    assert f'{container_info.image}:{container_info.tag}' in factory._client.images.image_list


def test_get_image_returns_correct_tag(factory, container_info):
    Images.add_image(f'{container_info.image}:{container_info.tag}')
    Images.add_image(f'{container_info.image}:other-tag')
    result = factory.get_image(container_info, quiet=True)
    assert result == f'{container_info.image}:{container_info.tag}'


def test_get_container_uses_correct_image(factory, source_no_build, monkeypatch):
    monkeypatch.setattr('tempfile.mkdtemp', lambda *args, **kwargs: 'TEMP_DIR')
    monkeypatch.setattr('shutil.copy', lambda *args, **kwargs: '')
    result = factory.get_container(source_no_build)
    assert result.image == 'python:3.7-alpine'


def test_get_container_runs_container_with_correct_settings(factory, source_no_build, monkeypatch):
    monkeypatch.setattr('tempfile.mkdtemp', lambda *args, **kwargs: 'TEMP_DIR')
    monkeypatch.setattr('shutil.copy', lambda *args, **kwargs: '')
    result = factory.get_container(source_no_build)
    assert result.name.startswith(source_no_build.name)
    assert result['command'] == 'sleep 1h'
    assert result['working_dir'] == '/src'
    assert result['detach']


def test_get_container_builds_correct_volume_info(factory, source_no_build, monkeypatch):
    monkeypatch.setattr('shutil.copy', lambda *args, **kwargs: '')
    monkeypatch.setattr('tempfile.mkdtemp', lambda *args, **kwargs: 'TEMP_DIR')
    result = factory.get_container(source_no_build)
    assert result['volumes'] == {'TEMP_DIR': {'bind': '/src', 'mode': 'rw'}}


def test_cleanup_removes_container(source_no_build, factory, no_io):
    container = factory.get_container(source_no_build)
    factory.cleanup(source_no_build)
    assert Containers.container_list[container.name].removed


def test_cleanup_removes_volume_dir(source_no_build, factory, no_io):
    def verify_rmtree(path, ignore_errors=False, *args, **kwargs):
        assert path == 'TEMP_DIR'
        assert ignore_errors
    factory.get_container(source_no_build)
    factory.cleanup(source_no_build)



from setuptools import setup, find_packages

setup(
    name='glotter',
    version='0.0.39',
    entry_points={'console_scripts': ['glotter = glotter.__main__:main']},
    packages=find_packages(exclude=('test',)),
    install_requires=[
        'docker>=3.7.0, <3.8',
        'Jinja2>=2.10.1, <2.11',
        'pytest>=4.3.1, <4.4',
        'PyYAML>=5.1, <5.2'
    ],
    python_requires='>=3.5',
)



import shutil
import tempfile
import pytest

from enum import Enum, auto

from glotter.project import Project


@pytest.fixture
def glotter_yml():
    return """
projects:
  baklava:
    words:
      - "baklava"
    requires_parameters: false
  fileio:
    words:
      - "file"
      - "io"
    requires_parameters: false
    acronyms:
      - "io"
  fibonacci:
    words:
      - "fibonacci"
    requires_parameters: true
  helloworld:
    words:
      - "hello"
      - "world"
    requires_parameters: false
"""


@pytest.fixture
def glotter_yml_projects():
    return {
        MockProjectEnum.Baklava: Project(
            words=["baklava"],
            requires_parameters=False,
        ),
        MockProjectEnum.FileIO: Project(
            words=["file", "io"],
            requires_parameters=False,
            acronyms=["io"]
        ),
        MockProjectEnum.Fibonacci: Project(
            words=["fibonacci"],
            requires_parameters=True
        ),
        MockProjectEnum.HelloWorld: Project(
            words=["hello", "world"],
            requires_parameters=False
        ),
    }


@pytest.fixture
def test_info_string_no_build():
    return """folder:
  extension: ".py"
  naming: "underscore"

container:
  image: "python"
  tag: "3.7-alpine"
  cmd: "python {{ source.name }}{{ source.extension }}"
"""


@pytest.fixture
def test_info_string_with_build():
    return """folder:
  extension: ".go"
  naming: "hyphen"

container:
  image: "golang"
  tag: "1.12-alpine"
  build: "go build -o {{ source.name }} {{ source.name}}{{ source.extension }}"
  cmd: "./{{ source.name }}"
"""


@pytest.fixture
def tmp_dir():
    dir = tempfile.mkdtemp()
    yield dir
    shutil.rmtree(dir, ignore_errors=True)


class MockProjectEnum(Enum):
    Baklava = auto()
    FileIO = auto()
    Fibonacci = auto()
    HelloWorld = auto()


@pytest.fixture
def patch_projects_enum(monkeypatch):
    monkeypatch.setattr('glotter.settings.Settings.projects_enum', MockProjectEnum)




import os
import pytest

from glotter.source import Source
from glotter.testinfo import TestInfo
from test.unit.fixtures import test_info_string_no_build, test_info_string_with_build, factory, docker, no_io, source_no_build, source_with_build


def test_full_path(test_info_string_no_build):
    src = Source('name', os.path.join('this', 'is', 'a', 'path'), test_info_string_no_build)
    expected = os.path.join('this', 'is', 'a', 'path', 'name')
    actual = src.full_path
    assert actual == expected


@pytest.mark.parametrize(('name', 'expected'), [
    ('name', ''),
    ('name.ext', '.ext'),
    ('name.name2.ext', '.ext')
])
def test_name(name, expected, test_info_string_no_build):
    src = Source(name, os.path.join('this', 'is', 'a', 'path'), test_info_string_no_build)
    actual = src.extension
    assert actual == expected


def test_test_info_matches_test_info_string(test_info_string_no_build):
    src = Source('name', os.path.join('this', 'is', 'a', 'path'), test_info_string_no_build)
    expected = TestInfo.from_string(test_info_string_no_build, src)
    actual = src.test_info
    assert actual == expected


def test_build_does_nothing_when_build_is_empty(test_info_string_no_build, monkeypatch):
    monkeypatch.setattr('glotter.containerfactory.ContainerFactory.get_container',
                        lambda *args, **kwargs: pytest.fail('get_container was called'))
    src = Source('name', os.path.join('this', 'is', 'a', 'path'), test_info_string_no_build)
    src.build()


def test_build_runs_build_command(factory, source_with_build, monkeypatch, no_io):
    source_with_build.build()
    build_cmd = source_with_build.test_info.container_info.build.strip()
    container = factory.get_container(source_with_build)
    actual = container.execs[0]
    assert actual.cmd.strip() == build_cmd.strip()
    assert not actual['detach']
    assert actual['workdir'] == '/src'


def test_build_raises_error_on_non_zero_exit_code_from_exec(source_with_build, monkeypatch, no_io):
    monkeypatch.setattr('glotter.source.Source._container_exec',
                        lambda *args, **kwargs: (1, 'error message'.encode('utf-8')))
    with pytest.raises(RuntimeError):
        source_with_build.build()


def test_run_execs_run_command(factory, source_no_build, no_io):
    source_no_build.run()
    run_cmd = source_no_build.test_info.container_info.cmd.strip()
    container = factory.get_container(source_no_build)
    actual = container.execs[0]
    assert actual.cmd.strip() == run_cmd.strip()
    assert not actual['detach']
    assert actual['workdir'] == '/src'


def test_run_execs_run_command_with_params(factory, source_no_build, no_io):
    params='-p param1 --longparam'
    source_no_build.run(params=params)
    run_cmd = f'{source_no_build.test_info.container_info.cmd.strip()} {params}'
    container = factory.get_container(source_no_build)
    actual = container.execs[0]
    assert actual.cmd.strip() == run_cmd.strip()
    assert not actual['detach']
    assert actual['workdir'] == '/src'


def test_run_on_non_zero_exit_code_from_exec_raises_no_error(source_no_build, monkeypatch, no_io):
    monkeypatch.setattr('glotter.source.Source._container_exec',
                        lambda *args, **kwargs: (1, 'error message'.encode('utf-8')))
    try:
        source_no_build.run()
    except Exception as e:
        pytest.fail(f'unexpected exception was thrown: {e}')


def test_exec_runs_command(factory, source_no_build, no_io):
    exec_cmd = 'command -p param1 --longparam'
    source_no_build.exec(exec_cmd)
    container = factory.get_container(source_no_build)
    actual = container.execs[0]
    assert actual.cmd.strip() == exec_cmd.strip()
    assert not actual['detach']
    assert actual['workdir'] == '/src'


def test_exec_on_non_zero_exit_code_raises_no_error(source_no_build, monkeypatch, no_io):
    exec_cmd = 'command -p param1 --longparam'
    monkeypatch.setattr('glotter.source.Source._container_exec',
                        lambda *args, **kwargs: (1, 'error message'.encode('utf-8')))
    try:
        source_no_build.exec(exec_cmd)
    except Exception as e:
        pytest.fail(f'unexpected exception was thrown: {e}')
